@@ -1,7 +1,10 @@
 """
 Mappings from file extensions to biopython types
 """
+import bz2
+import gzip
 import os.path
+import sys
 
 # Define mappings in a dictionary with extension : BioPython_file_type.
 EXTENSION_TO_TYPE = {'.aln': 'clustal',
@@ -21,11 +24,13 @@ EXTENSION_TO_TYPE = {'.aln': 'clustal',
                      '.phylip': 'phylip',
                      '.phyx': 'phylip-relaxed',
                      '.qual': 'qual',
-                     '.sff': 'sff',
+                     '.sff': 'sff-trim',
                      '.sth': 'stockholm',
                      '.sto': 'stockholm',
                      }
 
+COMPRESS_EXT = {'.bz2': bz2.BZ2File, '.gz': gzip.open, '.bz': bz2.BZ2File}
+
 
 class UnknownExtensionError(ValueError):
     pass
@@ -35,7 +40,7 @@ def from_extension(extension):
     """
     Look up the BioPython file type corresponding with input extension.
 
-    Lookup is case insensitive; the extension is presumed to start with a '.'
+    Look up is case insensitive.
     """
     if not extension.startswith('.'):
         raise ValueError("Extensions must begin with a period.")
@@ -48,7 +53,20 @@ def from_extension(extension):
 
 def from_filename(file_name):
     """
-    Lookup the BioPython file type corresponding to an input file name.
+    Look up the BioPython file type corresponding to an input file name.
     """
-    extension = os.path.splitext(file_name)[1]
+    base, extension = os.path.splitext(file_name)
+    if extension in COMPRESS_EXT:
+        # Compressed file
+        extension = os.path.splitext(base)[1]
     return from_extension(extension)
+
+def from_handle(fh, stream_default='fasta'):
+    """
+    Look up the BioPython file type corresponding to a file-like object.
+
+    For stdin, stdout, and stderr, ``stream_default`` is used.
+    """
+    if fh in (sys.stdin, sys.stdout, sys.stderr):
+        return stream_default
+    return from_filename(fh.name)
