@@ -1,7 +1,6 @@
 """
 Extract the sequence IDs from a file
 """
-import argparse
 import sys
 
 from Bio import SeqIO
@@ -12,10 +11,11 @@ from . import common
 
 def build_parser(parser):
     parser.add_argument('sequence_file', help="Sequence file",
-            type=argparse.FileType('r'))
+            type=common.FileType('r'))
     parser.add_argument('-o', '--output-file', help="Destination trimmed file",
-            type=argparse.FileType('w'), default=sys.stdout)
-    parser.add_argument('--source-format', default=None)
+            type=common.FileType('w'), default=sys.stdout)
+    parser.add_argument('--input-format', help="""Input format for sequence
+            file""")
     parser.add_argument('-d', '--include-description', action='store_true',
             default=False, help="""Include the sequence description in output
             [default: %(default)s]""")
@@ -24,8 +24,8 @@ def action(arguments):
     common.exit_on_sigpipe()
 
     # Determine file format for input and output
-    source_format = (arguments.source_format or
-            fileformat.from_filename(arguments.sequence_file.name))
+    source_format = (arguments.input_format or
+            fileformat.from_handle(arguments.sequence_file))
 
     with arguments.sequence_file:
         sequences = SeqIO.parse(arguments.sequence_file, source_format)
