@@ -2,9 +2,7 @@
 Modify sequence file(s) in place.
 """
 
-import argparse
 import logging
-import os.path
 
 from . import convert, common
 
@@ -15,7 +13,7 @@ def build_parser(parser):
 
     parser.add_argument(
         'input_files', metavar="sequence_file", nargs='+',
-        type=argparse.FileType('r+'),
+        type=common.FileType('r'),
         help="Sequence file(s) to mogrify")
 
     return parser
@@ -29,5 +27,6 @@ def action(arguments):
     for input_file in arguments.input_files:
         logging.info(input_file)
         # Generate a temporary file
-        with common.atomic_write(input_file.name) as tf:
+        with common.atomic_write(input_file.name,
+                file_factory=common.FileType('w')) as tf:
             convert.transform_file(input_file, tf, arguments)
