@@ -2,20 +2,24 @@
 Filter reads based on quality scores
 """
 
-import argparse
 import collections
 import csv
 import itertools
-import re
+import logging
+import os
 import sys
-from Queue import Queue, Empty
-import threading
+import time
 
 from Bio import SeqIO
+try:
+    from Bio import trie, triefind
+except ImportError:
+    trie = None
+    triefind = None
 from Bio.SeqIO import QualityIO
 
-from seqmagick import fileformat
-from .common import typed_range
+from seqmagick import fileformat, __version__
+from .common import typed_range, FileType
 
 # Default minimummean quality score
 DEFAULT_MEAN_SCORE = 25.0
@@ -23,45 +27,51 @@ DEFAULT_MEAN_SCORE = 25.0
 # Tools for working with ambiguous bases
 # Map from Ambiguous Base to regex
 _AMBIGUOUS_MAP = {
-       'R': '[GA]',
-       'Y': '[TC]',
-       'K': '[GT]',
-       'M': '[AC]',
-       'S': '[GC]',
-       'W': '[AT]',
-       'B': '[GTC]',
-       'D': '[GAT]',
-       'H': '[ACT]',
-       'V': '[GCA]',
-       'N': '[AGCT]',
+       'R': 'GA',
+       'Y': 'TC',
+       'K': 'GT',
+       'M': 'AC',
+       'S': 'GC',
+       'W': 'AT',
+       'B': 'GTC',
+       'D': 'GAT',
+       'H': 'ACT',
+       'V': 'GCA',
+       'N': 'AGCT',
 }
 
-def _ambiguous_pattern(sequence_str):
+def all_unambiguous(sequence_str):
     """
-    Convert an IUPAC ambiguous string to a regular expression pattern string
+    All unambiguous versions of sequence_str
     """
-    return ''.join(_AMBIGUOUS_MAP.get(c, c) for c in sequence_str)
-
+    result = [[]]
+    for c in sequence_str:
+        result = [i + [a] for i in result
+                  for a in _AMBIGUOUS_MAP.get(c, c)]
+    return [''.join(i) for i in result]
 
 def build_parser(parser):
     """
     Generate a subparser
     """
-    parser.add_argument('input_fastq', type=argparse.FileType('r'),
+    parser.add_argument('input_fastq', type=FileType('r'),
             help="""Input fastq file. A fasta-format file may also be provided
             if --input-qual is also specified.""")
-    parser.add_argument('--input-qual', type=argparse.FileType('r'),
+    parser.add_argument('--input-qual', type=FileType('r'),
             help="""The quality scores associated with the input file. Only
             used if input file is fasta.""")
-    parser.add_argument('output_file', type=argparse.FileType('w'),
+    parser.add_argument('output_file', type=FileType('w'),
             help="""Output file. Format determined from extension.""")
-    output_group = parser.add_argument_group("Output")
 
-    output_group.add_argument('--report-out', type=argparse.FileType('w'),
+    output_group = parser.add_argument_group("Output")
+    output_group.add_argument('--report-out', type=FileType('w'),
             default=sys.stdout, help="""Output file for report [default:
             stdout]""")
-    output_group.add_argument('--failure-out', type=argparse.FileType('w'),
-            help="""File to write failure report [default: None]""")
+    output_group.add_argument('--details-out', type=FileType('w'),
+             help="""Output file to report fate of each sequence""")
+    output_group.add_argument('--no-details-comment', action='store_false',
+            default=True, dest='details_comment', help="""Do not write comment
+            lines with version and call to start --details-out""")
 
     parser.add_argument('--min-mean-quality', metavar='QUALITY', type=float,
             default=DEFAULT_MEAN_SCORE, help="""Minimum mean quality score for
@@ -96,15 +106,21 @@ def build_parser(parser):
             will be removed.""", type=int)
 
     barcode_group = parser.add_argument_group('Barcode/Primer')
-    barcode_group.add_argument('--primer', help="""IUPAC ambiguous primer to
+    primer_group = barcode_group.add_mutually_exclusive_group()
+    primer_group.add_argument('--primer', help="""IUPAC ambiguous primer to
             require""")
-    barcode_group.add_argument('--barcode-file', help="""CSV file
-            containing sample_id,barcode rows""", type=argparse.FileType('r'))
+    primer_group.add_argument('--no-primer', help="""Do not use a primer.""",
+            action='store_const', const='', dest='primer')
+    barcode_group.add_argument('--barcode-file', help="""CSV file containing
+            sample_id,barcode[,primer] in the rows. A single primer for all
+            sequences may be specified with `--primer`, or `--no-primer` may be
+            used to indicate barcodes should be used without a primer
+            check.""", type=FileType('r'))
     barcode_group.add_argument('--barcode-header', action='store_true',
             default=False, help="""Barcodes have a header row [default:
             %(default)s]""")
     barcode_group.add_argument('--map-out', help="""Path to write
-            sequence_id,sample_id pairs""", type=argparse.FileType('w'),
+            sequence_id,sample_id pairs""", type=FileType('w'),
             metavar='SAMPLE_MAP')
     barcode_group.add_argument('--quoting', help="""A string naming an
             attribute of the csv module defining the quoting behavior for
@@ -133,82 +149,167 @@ def moving_average(iterable, n):
         d.append(elem)
         yield s / float(n)
 
-class FailureReportWriter(threading.Thread):
+class FailedFilter(Exception):
     """
-    Writes a log of sequences that failed filtering, and the filter that
-    removed them.
-    """
-    def __init__(self, queue, fp):
-        super(FailureReportWriter, self).__init__()
-        self.queue = queue
-        self.writer = csv.DictWriter(fp, ('failed_sequence', 'reason', 'value'),
-                delimiter='\t', lineterminator='\n')
-        self.writer.writeheader()
-
-    def run(self):
-        while True:
-            try:
-                record = self.queue.get(timeout=1)
-            except Empty:
-                continue
-
-            if record is None:
-                self.queue.task_done()
-                return
-            self.writer.writerow(record)
-            self.queue.task_done()
-
-
-class Failure(object):
-    """
-    Simple object to hold a value corresponding to a failure.  Evaluates to
-    false in boolean tests.
+    A read failed filtering
     """
     def __init__(self, value=None):
         self.value = value
 
-    def __nonzero__(self):
-        return False
+class RecordEventListener(object):
+    """
+    Contains and dispatches to handlers on events around sequence records
+
+    Event handlers take a single positional argument, the record, and optional
+    additional keyword arguments.
+    """
+    def __init__(self):
+        self.listeners = collections.defaultdict(set)
+
+    def __call__(self, event, record, **kwargs):
+        """
+        Trigger an event
+
+        :param event: Event name
+        :param record: Record affected
+        :param **kwargs: Optional additional arguments to pass to handlers
+        """
+        if event in self.listeners:
+            for listener in self.listeners[event]:
+                listener(record, **kwargs)
+
+    def register_handler(self, event, handler):
+        """
+        Register ``handler`` for ``event``
+        """
+        self.listeners[event].add(handler)
+
+    def iterable_hook(self, name, iterable):
+        """
+        Fire an event named ``name`` with each item in iterable
+        """
+        for record in iterable:
+            self(name, record)
+            yield record
+
+class RecordReportHandler(object):
+    """
+    Generates a report to a CSV file detailing every record processed.
+
+    Listens for events: [read, write, failed_filter, found_barcode]
+    """
+    HEADERS = ('sequence_name', 'in_length', 'in_mean_qual', 'sample',
+               'out_length', 'out_mean_qual', 'fail_filter', 'fail_value')
+    def __init__(self, fp, args, write_comments=True):
+        if write_comments:
+            fp.write('# Generated by `seqmagick quality-filter` version {0}\n'.format(__version__))
+            fp.write('# Arguments: {0}\n'.format(' '.join(args)))
+            fp.write('# Working directory: {0}\n'.format(os.getcwd()))
+
+        self.writer = csv.DictWriter(fp, self.HEADERS, lineterminator='\n',
+                quoting=csv.QUOTE_NONNUMERIC)
+        self.writer.writeheader()
+        self.current_record = None
+
+        self.read = 0
+        self.failed = 0
+        self.start = time.time()
+        self.last_report = 0.0
+
+    def register_with(self, listener):
+        listener.register_handler('failed_filter', self._record_failed)
+        listener.register_handler('read', self._read_record)
+        listener.register_handler('write', self._wrote_record)
+        listener.register_handler('found_barcode', self._found_barcode)
+
+    def _write(self):
+        assert self.current_record
+        self.writer.writerow(self.current_record)
+        self.current_record = None
+
+    def _record_failed(self, record, filter_name, value=None):
+        self.current_record.update({'fail_filter': filter_name,
+                                    'fail_value': value})
+
+        self._write()
+        self.failed += 1
+        self._report()
+
+    def _read_record(self, record):
+        self.current_record = {'sequence_name': record.id,
+                               'in_length': len(record)}
+        if 'phred_quality' in record.letter_annotations:
+            self.current_record['in_mean_qual'] = \
+                    mean(record.letter_annotations['phred_quality'])
+        self.read += 1
+
+    def _found_barcode(self, record, sample, barcode=None):
+        """Hook called when barcode is found"""
+        assert record.id == self.current_record['sequence_name']
+        self.current_record['sample'] = sample
+
+    def _wrote_record(self, record):
+        self.current_record['out_length'] = len(record)
+        if 'phred_quality' in record.letter_annotations:
+            self.current_record['out_mean_qual'] = \
+                    mean(record.letter_annotations['phred_quality'])
+        self._write()
+        self._report()
+
+    def _report(self):
+        if not sys.stdout.isatty():
+            return
+        t = time.time()
+        if t - self.last_report < 0.4 or not self.read:
+            return
+
+        self.last_report = t
+        sys.stderr.write('{0:10.1f}s Processed {1:10d} records; {2:10d} passed ({3:6.2f}%)\r'.format(
+            t - self.start,
+            self.read,
+            self.read - self.failed,
+            float(self.read - self.failed) / self.read * 100.0))
+
 
 class BaseFilter(object):
     """
     Base class for filters
     """
-    report_fields = ['name', 'passed_unchanged', 'passed_changed', 'failed',
-            'total_filtered', 'proportion_passed']
+    report_fields = ('name', 'passed_unchanged', 'passed_changed', 'failed',
+                     'total_filtered', 'proportion_passed')
 
-    def __init__(self):
+    def __init__(self, listener=None):
         self.passed_unchanged = 0
         self.passed_changed = 0
         self.failed = 0
+        self.listener = listener
 
     def filter_record(self, record):
         """
         Filter a record. If the filter succeeds, returns a SeqRecord. If it
-        fails, returns either None or an instance of Failure containing the
-        value with failed.
+        fails, raises an instance of FailedFilter with an optional value.
         """
         raise NotImplementedError("Override in subclass")
 
-    def filter_records(self, records, failure_queue=None):
+    def filter_records(self, records):
         """
         Apply the filter to records
         """
         for record in records:
-            filtered = self.filter_record(record)
-            if filtered:
+            try:
+                filtered = self.filter_record(record)
+                assert(filtered)
                 # Quick tracking whether the sequence was modified
                 if filtered == record:
                     self.passed_unchanged += 1
                 else:
                     self.passed_changed += 1
                 yield filtered
-            else:
+            except FailedFilter as e:
                 self.failed += 1
-                if failure_queue:
-                    value = filtered.value if filtered is not None else None
-                    failure_queue.put({'failed_sequence': record.id,
-                        'reason': self.name, 'value': value})
+                v = e.value
+                if self.listener:
+                    self.listener('failed_filter', record, filter_name=self.name, value=v)
 
     @property
     def passed(self):
@@ -227,7 +328,6 @@ class BaseFilter(object):
     def report_dict(self):
         return dict((f, getattr(self, f)) for f in self.report_fields)
 
-
 class QualityScoreFilter(BaseFilter):
     """
     Quality score filter - requires that the average base quality over the
@@ -242,13 +342,14 @@ class QualityScoreFilter(BaseFilter):
     def filter_record(self, record):
         """
         Filter a single record
-
-        Returns None if the record failed.
         """
         quality_scores = record.letter_annotations['phred_quality']
 
         mean_score = mean(quality_scores)
-        return record if mean_score >= self.min_mean_score else Failure(mean_score)
+        if mean_score >= self.min_mean_score:
+            return record
+        else:
+            raise FailedFilter(mean_score)
 
 class WindowQualityScoreFilter(BaseFilter):
     """
@@ -267,15 +368,16 @@ class WindowQualityScoreFilter(BaseFilter):
     def filter_record(self, record):
         """
         Filter a single record
-
-        Returns None if the record failed.
         """
         quality_scores = record.letter_annotations['phred_quality']
 
         # Simple case - window covers whole sequence
         if len(record) <= self.window_size:
             mean_score = mean(quality_scores)
-            return record if mean_score >= self.min_mean_score else Failure(mean_score)
+            if mean_score >= self.min_mean_score:
+                return record
+            else:
+                raise FailedFilter(mean_score)
 
         # Find the right clipping point. Start clipping at the beginning of the
         # sequence, then extend the window to include regions with acceptable
@@ -291,6 +393,9 @@ class WindowQualityScoreFilter(BaseFilter):
 
         if clip_right:
             return record[:clip_right]
+        else:
+            # First window failed - record fails
+            raise FailedFilter()
 
 class AmbiguousBaseFilter(BaseFilter):
     """
@@ -318,7 +423,7 @@ class AmbiguousBaseFilter(BaseFilter):
         elif self.action == 'truncate':
             return record[:nloc]
         elif self.action == 'drop':
-            return None
+            raise FailedFilter()
         else:
             assert False
 
@@ -337,7 +442,7 @@ class MaxAmbiguousFilter(BaseFilter):
     def filter_record(self, record):
         n_count = record.seq.upper().count('N')
         if n_count > self.max_ambiguous:
-            return Failure(n_count)
+            raise FailedFilter(n_count)
         else:
             assert n_count <= self.max_ambiguous
             return record
@@ -360,7 +465,7 @@ class MinLengthFilter(BaseFilter):
         if l >= self.min_length:
             return record
         else:
-            return Failure(l)
+            raise FailedFilter(l)
 
 class MaxLengthFilter(BaseFilter):
     """
@@ -393,42 +498,31 @@ class PrimerBarcodeFilter(BaseFilter):
     """
     name = "Primer/Barcode"
 
-    def __init__(self, primer, barcodes, output_file=None, trim=True, quoting=csv.QUOTE_MINIMAL):
+    def __init__(self, trie, output_file=None, trim=True, quoting=csv.QUOTE_MINIMAL):
         super(PrimerBarcodeFilter, self).__init__()
-        self.primer = primer
-        self.barcodes = barcodes
         self.trim = True
-        if output_file:
-            self.writer = csv.writer(output_file, lineterminator='\n', quoting = quoting)
-        else:
-            self.writer = None
-
-        self.pattern = re.compile('^({0}){1}'.format('|'.join(barcodes),
-                                                    _ambiguous_pattern(primer)),
-                                  re.IGNORECASE)
-
-    def _report_match(self, record, sample):
-        if not self.writer:
-            return
-        self.writer.writerow((record.id, sample))
+        self.trie = trie
 
     def filter_record(self, record):
-        m = self.pattern.match(str(record.seq))
+        m = triefind.match(str(record.seq), self.trie)
         if m:
-            self._report_match(record, self.barcodes[m.group(1)])
+            if self.listener:
+                self.listener('found_barcode', record, barcode=m, sample=self.trie[m])
             if self.trim:
-                record = record[m.end():]
+                record = record[len(m):]
             return record
+        else:
+            raise FailedFilter()
 
-def parse_barcode_file(fp, header=False):
+def parse_barcode_file(fp, primer=None, header=False):
     """
-    Load label, barcode pairs from a CSV file.
+    Load label, barcode, primer records from a CSV file.
 
     Returns a map from barcode -> label
 
     Any additional columns are ignored
     """
-    d = {}
+    tr = trie.trie()
     reader = csv.reader(fp)
 
     if header:
@@ -439,24 +533,19 @@ def parse_barcode_file(fp, header=False):
     records = (record for record in reader if record)
 
     for record in records:
-        value, key = record[:2]
-        if key in d:
-            raise ValueError("Duplicate barcode: {0}".format(key))
-        if value in d.values():
-            raise ValueError("Duplicate sample label: {0}".format(value))
-        d[key] = value
+        specimen, barcode = record[:2]
+        if primer is not None:
+            pr = primer
+        else:
+            pr = record[2]
+        for sequence in all_unambiguous(barcode + pr):
+            if tr.has_key(sequence):
+                raise ValueError("Duplicate sample: {0}, {1} both have {2}",
+                        specimen, tr[sequence], sequence)
+            logging.info('%s->%s', sequence, specimen)
+            tr[sequence] = specimen
 
-    # Require that all barcodes are the same length
-    if min(len(k) for k in d) != max(len(k) for k in d):
-        keys = d.keys()
-        keys.sort(key=len)
-        minlen_key = keys[0]
-        maxlen_key = keys[-1]
-
-        raise ValueError(("Length of barcode '{0}' does not match "
-            "length of barcode '{1}'").format(minlen_key, maxlen_key))
-
-    return d
+    return tr
 
 def action(arguments):
     """
@@ -466,18 +555,14 @@ def action(arguments):
         raise ValueError("--quality-window-mean-qual specified without "
                 "--quality-window")
 
-    queue = None
-    if arguments.failure_out:
-        queue = Queue()
-        t = FailureReportWriter(queue, arguments.failure_out)
-        t.setDaemon(True)
-        t.start()
+    if trie is None or triefind is None:
+        raise ValueError('Missing Bio.trie and/or Bio.triefind modules. Cannot continue')
 
     # Always filter with a quality score
     qfilter = QualityScoreFilter(arguments.min_mean_quality)
     filters = [qfilter]
 
-    output_type = fileformat.from_filename(arguments.output_file.name)
+    output_type = fileformat.from_handle(arguments.output_file)
     with arguments.input_fastq as fp:
         if arguments.input_qual:
             sequences = QualityIO.PairedFastaQualIterator(fp,
@@ -485,6 +570,15 @@ def action(arguments):
         else:
             sequences = SeqIO.parse(fp, 'fastq')
 
+        listener = RecordEventListener()
+        if arguments.details_out:
+            rh = RecordReportHandler(arguments.details_out, arguments.argv,
+                    arguments.details_comment)
+            rh.register_with(listener)
+
+        # Track read sequences
+        sequences = listener.iterable_hook('read', sequences)
+
         # Add filters
         if arguments.max_length:
             max_length_filter = MaxLengthFilter(arguments.max_length)
@@ -508,15 +602,24 @@ def action(arguments):
 
         if arguments.barcode_file:
             with arguments.barcode_file:
-                barcodes = parse_barcode_file(arguments.barcode_file,
-                        arguments.barcode_header)
-            f = PrimerBarcodeFilter(arguments.primer or '', barcodes,
-                    arguments.map_out,
-                    quoting=getattr(csv, arguments.quoting))
+                tr = parse_barcode_file(arguments.barcode_file,
+                        arguments.primer, arguments.barcode_header)
+            f = PrimerBarcodeFilter(tr)
             filters.append(f)
 
+            if arguments.map_out:
+                barcode_writer = csv.writer(arguments.map_out,
+                        quoting=getattr(csv, arguments.quoting),
+                        lineterminator='\n')
+                def barcode_handler(record, sample, barcode=None):
+                    barcode_writer.writerow((record.id, sample))
+                listener.register_handler('found_barcode', barcode_handler)
         for f in filters:
-            sequences = f.filter_records(sequences, queue)
+            f.listener = listener
+            sequences = f.filter_records(sequences)
+
+        # Track sequences which passed all filters
+        sequences = listener.iterable_hook('write', sequences)
 
         with arguments.output_file:
             SeqIO.write(sequences, arguments.output_file, output_type)
@@ -529,6 +632,3 @@ def action(arguments):
                 lineterminator='\n', delimiter='\t')
         writer.writeheader()
         writer.writerows(rpt_rows)
-
-    if queue:
-        queue.join()
