@@ -6,15 +6,11 @@ import shutil
 import tempfile
 
 from seqmagick.scripts import cli
+from seqmagick.subcommands.common import FileType
+from seqmagick.test.integration import data_path
 
 from . import test_convert
 
-d = os.path.dirname(__file__)
-data_dir = os.path.join(d, "data")
-
-def p(*args):
-    return os.path.join(data_dir, *args)
-
 class CommandLineTestMixIn(object):
     def setUp(self):
         with tempfile.NamedTemporaryFile(
@@ -31,14 +27,24 @@ class CommandLineTestMixIn(object):
         except SystemExit, e:
             self.fail(e)
 
-        with open(self.input_file) as fp:
+        with FileType('r')(self.input_file) as fp:
             actual = fp.read()
-        with open(self.expected_path) as fp:
+        with FileType('r')(self.expected_path) as fp:
             expected = fp.read()
         self.assertEqual(expected, actual)
 
     def tearDown(self):
         os.remove(self.input_file)
 
-class TestMogrifyUngapCut(CommandLineTestMixIn, test_convert.TestConvertUngapCut):
+class MogrifyUngapCutTestCase(CommandLineTestMixIn, test_convert.ConvertUngapCutTestCase):
     command = 'mogrify --ungap --cut 1:3 --tail 2 {input}'
+
+class MogrifyBzipInputTestCase(CommandLineTestMixIn, test_convert.BzipInputConvertTestCase):
+    command = 'mogrify {input}'
+    expected_path = data_path('output2.fasta')
+    out_suffix='fasta.bz2'
+
+class MogrifyGzipInputTestCase(CommandLineTestMixIn, test_convert.GzipInputConvertTestCase):
+    command = 'mogrify {input}'
+    expected_path = data_path('output2.fasta')
+    out_suffix='fasta.gz'
