@@ -3,12 +3,14 @@ Functions to transform / filter sequences
 """
 import collections
 import contextlib
+import copy
 import cPickle as pickle
 import itertools
 import logging
 import re
 import string
 import tempfile
+import random
 
 from Bio import SeqIO
 from Bio.Alphabet import IUPAC
@@ -465,6 +467,8 @@ def seq_exclude(records, filter_regex):
         if not regex.search(str(record.seq)):
             yield record
 
+def sample(records, sample):
+    return random.sample(list(records), sample)
 
 def head(records, head):
     """
@@ -628,6 +632,7 @@ def translate(records, translate):
              'rna': CodonTable.ambiguous_rna_by_name["Standard"]}[source_type]
 
     # Handle ambiguities by replacing ambiguous codons with 'X'
+    table = copy.deepcopy(table)
     table.forward_table = CodonWarningTable(table.forward_table)
 
     for record in records:
